-------------------------------------------------------------------------------
-- This is just a demo code, to show the fpgadbg functionality on the
-- Spartan 3 Starter Board
-- Written by Wojciech M. Zabolotny 27.07.2006
-- Inspired by MANY sources.
-- Consider it to be a PUBLIC DOMAIN code
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity fpgadbg_s3sb is
  
  port (
    digit                     : out std_logic_vector(3 downto 0);
    segment                   : out std_logic_vector(7 downto 0);
    Push_Buttons_3Bit_GPIO_in : in  std_logic_vector(2 downto 0);
    LEDs_8Bit_GPIO_d_out      : out std_logic_vector(7 downto 0);
    sys_rst                   : in  std_logic;
    RS232_RX                  : in  std_logic;
    RS232_TX                  : out std_logic;
    sys_clk                   : in  std_logic);

end fpgadbg_s3sb;

architecture arch1 of fpgadbg_s3sb is

  component fpgadbg_uart
    generic (
      brg_div     : integer;
      width       : integer;
      log2samples : integer);
    port (
      wr_clk  : in  std_logic;
      trigger : in  std_logic;
      data_in : in  std_logic_vector((width-1) downto 0);
      rs_rx   : in  std_logic;
      rs_tx   : out std_logic;
      --dbg     : out std_logic_vector(7 downto 0);
      sys_clk : in  std_logic;
      nrst    : in  std_logic);
  end component;

  type   bcdi is array(3 downto 0) of integer range 0 to 9;
  signal d : bcdi := (0, 0, 0, 0);      -- time counter (4 BCD digits)

  signal digcount : integer range 0 to 3 := 0;  -- digit counter
  type   dconv is array (9 downto 0) of std_logic_vector(7 downto 0);

  signal   trigger  : std_logic;
  signal   data_in  : std_logic_vector(32 downto 0);
  constant dig_conv : dconv :=
    ("11100110", "11111110", "11100000", "00111110",
     "10110110", "01100110", "11110010", "11011010",
     "01100000", "11111100");           -- conversion table for LED display

  signal freqdiv        : integer   := 0;    -- divider producing 100Hz clock
  signal freqdiv2       : integer   := 0;    -- LED display divider
  signal count, count_a : std_logic := '0';  -- timer's state
  --signal dbg            : std_logic_vector(7 downto 0);
  signal start          : std_logic;         -- start the timer
  signal stop           : std_logic;         -- stop the timer

begin  -- arch1

  disp1 : process (sys_clk, sys_rst)
  begin  -- process disp1
    if sys_rst = '1' then               -- asynchronous reset (active high)
      digcount <= 0;
    elsif sys_clk'event and sys_clk = '1' then  -- rising clock edge
      freqdiv2 <= freqdiv2 + 1;
      if freqdiv2 = 1000 then
        freqdiv2 <= 0;
        digcount <= digcount + 1;
        if digcount = 3 then
          digcount <= 0;
        end if;
      end if;
    end if;
  end process disp1;

  -- Translation of digits into the segments' state
  segment <= not dig_conv(d(digcount));

  -- Which digit should light?
  disp2 : for i in 0 to 3 generate
    digit(3-i) <= '0' when digcount = i else '1';
  end generate disp2;

  -- Buttons used to control the timer
  start <= Push_Buttons_3Bit_GPIO_in(0);
  stop  <= Push_Buttons_3Bit_GPIO_in(1);

  -- Process for starting and stopping of the timer
  st1 : process (start, stop, sys_rst)
  begin  -- process st1
    if sys_rst = '1' then
      count_a <= '0';
    elsif start = '1' then
      count_a <= '1';
    elsif stop = '1' then
      count_a <= '0';
    end if;
  end process st1;

  LEDs_8Bit_GPIO_d_out <= (others => '0');  -- Was dbg;

  -- Process for synchronization of the timer start and stop
  cnt1 : process (sys_clk, sys_rst)
  begin  -- process cnt1
    if sys_rst = '1' then               -- asynchronous reset (active high)
      count <= '0';
    elsif sys_clk'event and sys_clk = '1' then  -- rising clock edge
      count <= count_a;
    end if;
  end process cnt1;

  -- The main process of the timer
  process (sys_clk, sys_rst)
  begin  -- process
    if sys_rst = '1' then               -- asynchronous reset (active high)
      d(0) <= 0;
      d(1) <= 0;
      d(2) <= 0;
      d(3) <= 0;
    elsif sys_clk'event and sys_clk = '1' then  -- rising clock edge
      freqdiv <= freqdiv+1;
      if freqdiv = 500000 then
        freqdiv <= 0;
        if count = '1' then
          d(0) <= d(0) + 1;
          if d(0) = 9 then
            d(1) <= d(1) + 1;
            d(0) <= 0;
            if d(1) = 9 then
              d(2) <= d(2) + 1;
              d(1) <= 0;
              if d(2) = 9 then
                d(3) <= d(3) + 1;
                d(2) <= 0;
                if d(3) = 9 then
                  d(3) <= 0;
                end if;
              end if;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;

  -- Connection of the debug interface

  data_in(20 downto 0)  <= std_logic_vector(to_unsigned(freqdiv, 21));
  data_in(32 downto 21) <= std_logic_vector(to_unsigned(freqdiv2, 12));

  trigger <= '1' when d(1) = 1 else '0';

  fpgadbg_uart_1 : fpgadbg_uart
    generic map (
      brg_div     => 50000000/16/115200,
      width       => 33,
      log2samples => 10)
    port map (
      wr_clk  => sys_clk,
      trigger => trigger,
      data_in => data_in,
      rs_rx   => RS232_RX,
      rs_tx   => RS232_TX,
      --dbg     => dbg,
      sys_clk => sys_clk,
      nrst    => not sys_rst);
end arch1;
